package com.avaya.sdksampleapp.commpackage;

import android.app.Fragment;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ListView;

import com.avaya.clientservices.messaging.Message;

import java.util.ArrayList;
import java.util.List;

/**
 * ConversationFragment used to show list of the messages of the active conversation
 */
public class ConversationFragment extends Fragment {
    private final String LOG_TAG = this.getClass().getSimpleName();
    private List<Message> conversationList = new ArrayList<>();
    private ConversationItem item;
    private EditText messageEditText;
    private ChatMessagesListAdapter chatMessagesListAdapter;
    private MessagingManager messagingManager;

    private ConversationFragment.FragmentReceiver fragmentReceiver;

    public void setConversation(ConversationItem item) {
        this.item = item;
    }

    public void onCreate(Bundle savedInstanceState) {
        Log.d(LOG_TAG, "onCreate()");
        super.onCreate(savedInstanceState);
        messagingManager = SDKManager.getInstance(getActivity()).getMessagingManager();
        fragmentReceiver = new FragmentReceiver();
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        Log.d(LOG_TAG, "onCreateView()");
        return inflater.inflate(R.layout.conversation_fragment, container, false);
    }

    public void onViewCreated(View view, Bundle savedInstanceState) {
        Log.d(LOG_TAG, "onViewCreated()");
        fragmentReceiver.view = view;
        super.onViewCreated(view, savedInstanceState);
        messageEditText = (EditText) view.findViewById(R.id.editTextMessage);
        conversationList = item.getMessages();
        ListView conversationListView = (ListView) view.findViewById(R.id.messages_list);
//        // Initializing ListView adapter
        chatMessagesListAdapter = new ChatMessagesListAdapter(getActivity(), R.layout.chat_item, conversationList);
        conversationListView.setAdapter(chatMessagesListAdapter);
        item.setAdapter(chatMessagesListAdapter);
        Button sentButton = (Button) view.findViewById(R.id.send_message_button);
        sentButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String text = messageEditText.getText().toString();
                if (!text.isEmpty() && messagingManager != null) {
                    messagingManager.createMessage(item.getConversationItem().getId(), text);
                    messageEditText.getText().clear();
                    chatMessagesListAdapter.notifyDataSetChanged();
                }
            }
        });
    }

    @Override
    public void onStart() {
        Log.d(LOG_TAG, "onStart()");
        super.onStart();
    }

    @Override
    public void onDestroyView() {
        Log.d(LOG_TAG, "onDestroyView()");
        super.onDestroyView();

    }

    @Override
    public void onResume() {
        Log.d(LOG_TAG, "onResume()");
        super.onResume();
        // Set fragment title
        getActivity().setTitle("conversation with " + item.name);

        // Register broadcast receiver for video events when resume to the screen

        getActivity().registerReceiver(fragmentReceiver,
                new IntentFilter(SDKManager.MESSAGING_IS_TYPING_TURNED_ON));
        getActivity().registerReceiver(fragmentReceiver,
                new IntentFilter(SDKManager.MESSAGING_IS_TYPING_TURNED_OFF));
    }

    // Receiver of Broadcast messages.
    class FragmentReceiver extends BroadcastReceiver {

        public View view;

        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            CheckBox isTypingButton = (CheckBox) view.findViewById(R.id.is_typing);

            if(action.equals(SDKManager.MESSAGING_IS_TYPING_TURNED_ON)) {
                isTypingButton.setChecked(true);
            }
            else if(action.equals(SDKManager.MESSAGING_IS_TYPING_TURNED_OFF))
            {
                isTypingButton.setChecked(false);
            }

        }
    }
}

